@@ -52,9 +52,10 @@ ibwebadmin
 kernel-patch-badram
 kolab-webadmin
 kolabadmin
-gstreamer0.10-plugins-really-bad
+gstreamer.\+-plugins-really-bad
 liberrors.so
 liberrors-samba
+libsamba-errors
 gsambad
 libad
 libtest-nowarnings-perl
@@ -63,6 +64,7 @@ libclass-errorhandler-perl
 zope-ploneerrorreporting
 libroxen-errormessage
 liberror-perl
+perl-Error
 libgpg-error-dev
 libgpg-error0
 Opts:.\+errors=remount
@@ -80,7 +82,7 @@ deverror.out
 (floppy), sector 0
 mount version older than kernel
 Can't locate module
-Warning only 896MB will be used.
+Warning only .\+MB will be used.
 hostname: Host name lookup failure
 I can't tell the difference.
 warning, not much extra random data, consider using the -rand option
@@ -118,6 +120,7 @@ asking for cache data failed
 However, I can not read the target:
 Warning: The partition table looks like it was made
 task_error=0
+task_local_error=0
 ^info: Trying to set
 warning: /usr/lib/X11/fonts
 can't read /etc/udev/rules.d/z25_persistent-net.rules
@@ -150,16 +153,43 @@ can't initialize iptables table
 can't initialize ip6tables table
 Authentication warning overridden
 41-warning.sh
+PCCT header not found
+Download is performed unsandboxed as root as file
 update-alternatives: warning: skip creation of
 loop: module verification failed: signature
 Warning: apt-key output should not be parsed
 WARNING: Failed to connect to lvmetad. Falling back to device scanning
+Warning: The home dir /var/lib/usbmux you specified
+diff: /var/lib/apparmor/profiles/.apparmor.md5sums: No such file or directory
+error reporting disabled
+Enabling Firmware First mode for corrected errors
+errors: 0
+ 0 errors
+Memory Error Correction:
+Memory Controller 0 - Channel . Error
+IIO RAS/Control Status/Global Errors
+RAS: Correctable Errors collector initialized
+__stack_chk_fail
+grub.cfg.new: Directory nonexistent
+can't derive routing for PCI INT A
+failed to load isci/isci_firmware.bin
+Direct firmware load for isci/isci_firmware.bin failed with error
+Loading user firmware failed, using default values
+stunnel4 you specified can't be accessed: No such file or directory
+install-docs --verbose --check file_name' may give more details about the above errors
+cannot open '/etc/ssl/certs/java/cacerts' for reading: No such file or directory
+can't claim BAR
+disabling ASPM
+data block query control method not found
+subprocess.py.\+RuntimeWarning: line buffering
+Resource conflict.\+ found
 update-rc.d: warning: start and stop actions are no longer supported"
 
 # add pattern on some conditions
 if [ -n $FAI_ALLOW_UNSIGNED ] ; then
    globalignorepatterns="$globalignorepatterns
 WARNING: untrusted versions
+WARNING: The following packages cannot be authenticated
 Ignoring these trust violations"
 fi
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
