@@ -253,12 +253,7 @@ sub build_cryptsetup_commands {
           "", "keyfile_$real_dev" );
 
         my $lukscreateopts = $vol->{lukscreateopts} // "";
-        if ($lukscreateopts !~ /(^|\s)-c\s+\S+/) {
-          $lukscreateopts .= " -c aes-cbc-essiv:sha256";
-        }
-        if ($lukscreateopts !~ /(^|\s)-s\s+\d+/) {
-          $lukscreateopts .= " -s 256";
-        }
+
         # encrypt
         &FAI::push_command(
           "yes YES | cryptsetup luksFormat $real_dev $keyfile $lukscreateopts",
@@ -282,7 +277,7 @@ sub build_cryptsetup_commands {
         }
 
         # add entries to crypttab
-        push @FAI::crypttab, "$enc_dev_short_name\t$real_dev\t$keyfile\tluks";
+        push @FAI::crypttab, "$enc_dev_short_name\t$real_dev\t$keyfile\tluks,discard";
       } elsif ($mode eq "tmp" || $mode eq "swap") {
         &FAI::push_command(
           "cryptsetup --key-file=/dev/urandom create $enc_dev_short_name $real_dev",
@@ -323,6 +318,7 @@ sub build_btrfs_commands {
       next unless defined($this_fs);
       next unless ($this_fs eq 'btrfs');
       $volume = 'single_' . $single_vol_index;
+      $FAI::configs{$config}{volumes}{$volume}{preserve} = $FAI::configs{$c}{partitions}{$p}{size}{preserve};
       $FAI::configs{$config}{volumes}{$volume}{encrypt} = $FAI::configs{$c}{partitions}{$p}{encrypt};
       $FAI::configs{$config}{volumes}{$volume}{raidlevel} = 'single';
       $FAI::configs{$config}{volumes}{$volume}{filesystem} = $this_fs;
@@ -332,7 +328,7 @@ sub build_btrfs_commands {
       $FAI::configs{$config}{volumes}{$volume}{mount_options} = $FAI::configs{$c}{partitions}{$p}{mount_options};
       $FAI::configs{$c}{partitions}{$p}{mount_options} = '-';
       $FAI::configs{$config}{volumes}{$volume}{fstabkey} = $FAI::configs{$c}{fstabkey};
-      if ($device =~ m:^/dev/nvme|^/dev/loop:) {
+      if ($device =~ m:^/dev/nvme|^/dev/loop|^/dev/mmcblk:) {
         $FAI::configs{$config}{volumes}{$volume}{devices}{$device . "p" . $p} = {};
       } else {
         $FAI::configs{$config}{volumes}{$volume}{devices}{$device . $p} = {};
@@ -342,6 +338,8 @@ sub build_btrfs_commands {
     }
   }
 
+  my %mkfs_done;
+
   foreach my $config (keys %FAI::configs) { # loop through all configs
     next unless ($config eq "BTRFS");
 
@@ -360,6 +358,7 @@ sub build_btrfs_commands {
     my $createopts = $vol->{createopts} // "";
     $createopts .= " $forcebtrfs";
     my $pre_req = "";
+    my $btrfs_tool = "";
     # creates the proper prerequisites for later command ordering
     foreach (@devs) {
       my $tmp = $_;
@@ -369,17 +368,25 @@ sub build_btrfs_commands {
     if (scalar @devs == 1) {
       $pre_req = "exist_" . $devs[0];
     }
+
     # creates the BTRFS volume/RAID
     if ($raidlevel eq 'single') {
-          &FAI::push_command("mkfs.btrfs -d single $createopts ". join(" ",@devs),
-                             "$pre_req",
-                             "btrfs_built_raid_$id");
+      if (exists $mkfs_done{join(" ", @devs)}) {
+	$btrfs_tool = "true";
+      } else {
+	print "Adding mkfs command for '", join(", ", @devs), "'.\n";
+	$btrfs_tool = "mkfs.btrfs -d single $createopts ".join(" ",@devs);
+	$mkfs_done{join(" ", @devs)} = '1';
+      }
+    } else {
+      $btrfs_tool = "mkfs.btrfs -d raid$raidlevel $createopts ".join(" ",@devs);
+    }
 
-        } else {
-          &FAI::push_command("mkfs.btrfs -d raid$raidlevel $createopts ". join(" ",@devs),
-                             "$pre_req",
-                             "btrfs_built_raid_$id");
-        }
+    # nothing more to do if we need to proserve this volume. No mkfs, no subvolume
+    next if ($vol->{preserve});
+
+    # add mkfs.btrfs if needed (otherwise add true)
+    &FAI::push_command("$btrfs_tool", "$pre_req", "btrfs_built_raid_$id");
 
     # initial mount, required to create the initial subvolume
     &FAI::push_command("mount $devs[0] /mnt",
@@ -427,7 +434,7 @@ sub build_raid_commands {
 
   foreach my $config (keys %FAI::configs) { # loop through all configs
     # no encrypted, tmpfs, LVM or physical devices here
-    next if ($config eq "BTRFS" || $config eq "CRYPT" || $config eq "TMPFS" || $config =~ /^VG_./ || $config =~ /^PHY_./);
+    next if ($config eq "BTRFS" || $config eq "CRYPT" || $config eq "TMPFS" || $config eq "NFS" || $config =~ /^VG_./ || $config =~ /^PHY_./);
     ($config eq "RAID") or &FAI::internal_error("Invalid config $config");
 
     # create all raid devices
@@ -511,6 +518,14 @@ sub build_raid_commands {
           . join(" ", @eff_devs) . " " . join(" ", @spares),
         "$pre_req", "exist_/dev/md$id" );
 
+      # do not sync soft raid during installation
+      # do not remove ; from echo line. Otherwise it will not work
+      if ($FAI::do_init_tasks) { # no sync when initial installation
+	&FAI::push_command(
+          "echo frozen > /sys/block/md$id/md/sync_action;",
+          "exist_/dev/md$id", "nosync_md$id" );
+      }
+
       # create the filesystem on the volume
       &FAI::build_mkfs_commands("/dev/md$id",
         \%{ $FAI::configs{$config}{volumes}{$id} });
@@ -847,7 +862,7 @@ sub cleanup_vg {
   my $post_wipe = "pvremove_$vg";
   foreach my $d (split (" ", $devices)) {
     $post_wipe .= ",pv_sigs_removed_wipe_${d}_$vg";
-    &FAI::push_command( "wipefs -a $d", "pvremove_$vg", "pv_sigs_removed_wipe_${d}_$vg");
+    &FAI::push_command( "wipefs -af $d", "pvremove_$vg", "pv_sigs_removed_wipe_${d}_$vg");
   }
   &FAI::push_command( "true", $post_wipe, "pv_sigs_removed_$vg" );
   return 1;
@@ -886,7 +901,7 @@ sub build_lvm_commands {
     # prevent error due to different VG name of existing VG
     if (defined($FAI::configs{$d})) {
       # don't deactivate preserved VGs to prevent blkid error later on
-      foreach my $v (keys %{ $FAI::configs->{$d}->{volumes} }) {
+      foreach my $v (keys %{ $FAI::configs{$d}->{volumes} }) {
         $preserved = 1 if ($FAI::configs{$d}{volumes}{$v}{size}{preserve});
       }
     }
@@ -908,7 +923,7 @@ sub build_lvm_commands {
   foreach my $config (keys %FAI::configs) {
 
     # no physical devices, RAID, encrypted or tmpfs here
-    next if ($config eq "BTRFS" || $config =~ /^PHY_./ || $config eq "RAID" || $config eq "CRYPT" || $config eq "TMPFS");
+    next if ($config eq "BTRFS" || $config =~ /^PHY_./ || $config eq "RAID" || $config eq "CRYPT" || $config eq "TMPFS" || $config eq "NFS");
     ($config =~ /^VG_(.+)$/) or &FAI::internal_error("Invalid config $config");
     next if ($1 eq "--ANY--");
     my $vg = $1; # the volume group
@@ -1143,7 +1158,7 @@ sub setup_partitions {
       scalar(@{ $FAI::current_dev_children{$c} }));
     my ($i_p_d, $d, $part_no) = &FAI::phys_dev($c);
     ($i_p_d && $d eq $disk) or &FAI::internal_error("Invalid dev children entry");
-    my $wipe_cmd = "wipefs -a $c";
+    my $wipe_cmd = "wipefs -af $c";
     foreach my $part_id (@to_preserve) {
       # get the existing id
       my $mapped_id = $FAI::configs{$config}{partitions}{$part_id}{maps_to_existing};
@@ -1304,6 +1319,10 @@ sub setup_partitions {
     if (defined $part->{mountpoint} && $part->{mountpoint} eq "/boot") {
       $boot_disk=$disk;
     }
+    # override $boot_disk if /boot/efi exists
+    if (defined $part->{mountpoint} && $part->{mountpoint} eq "/boot/efi") {
+      $boot_disk=$disk;
+    }
 
     # the type of the partition defaults to primary
     my $part_type = "primary";
@@ -1354,9 +1373,10 @@ sub setup_partitions {
     $prev_id = $part_id;
   }
 
+  # set bootable flag for gpt-bios and gpt
   &FAI::push_command("parted $boot_disk set 1 boot on",
     "pt_complete_$disk", "gpt_bios_fake_bootable")
-    if($FAI::configs{$config}{disklabel} eq "gpt-bios" and $boot_disk);
+    if($FAI::configs{$config}{disklabel} =~ /^gpt/ and $boot_disk);
 
   ($prev_id > -1) or &FAI::internal_error("No partitions created");
   $partition_table_deps{$disk} = "cleared2_$disk,exist_"
@@ -1375,7 +1395,7 @@ sub build_disk_commands {
   # loop through all configs
   foreach my $config ( keys %FAI::configs ) {
     # no RAID, encrypted, tmpfs or LVM devices here
-    next if ($config eq "BTRFS" || $config eq "RAID" || $config eq "CRYPT" || $config eq "TMPFS" || $config =~ /^VG_./ || $FAI::configs{$config}{vg});
+    next if ($config eq "BTRFS" || $config eq "RAID" || $config eq "CRYPT" || $config eq "TMPFS" || $config eq "NFS" || $config =~ /^VG_./ || $FAI::configs{$config}{vg});
     ($config =~ /^PHY_(.+)$/) or &FAI::internal_error("Invalid config $config");
     my $disk = $1; # the device to be configured
 
