@@ -178,7 +178,7 @@ sub init_part_config {
   # only supported on msdos disk labels.
   ($FAI::device =~ /^PHY_(.+)$/ && ($type ne "logical"
       || $FAI::configs{$FAI::device}{disklabel} eq "msdos")) or
-    die "Syntax error: invalid partition type";
+    die "Syntax error: invalid partition type. A logical partition is only supported with msdos disk labels.";
 
   # the disk
   my $disk = $1;
@@ -492,6 +492,12 @@ $FAI::Parser = Parse::RecDescent->new(
           $FAI::configs{$FAI::device}{fstabkey} = "device";
           $FAI::configs{$FAI::device}{volumes} = {};
         }
+        | /^nfs/
+        {
+          $FAI::device = "NFS";
+          $FAI::configs{$FAI::device}{fstabkey} = "device";
+          $FAI::configs{$FAI::device}{volumes} = {};
+        }
         | /^disk(\d+)/
         {
           # check, whether parted is available
@@ -771,7 +777,7 @@ $FAI::Parser = Parse::RecDescent->new(
           $FAI::partition_pointer_dev_name = "/dev/md$vol_id";
         }
         mountpoint devices filesystem mount_options mdcreateopts
-        | /^btrfs (single|raid([0156]|10))\s+/
+        | /^btrfs\s+(single|raid([0156]|10))\s+/
         {
           ($FAI::device eq "BTRFS") or die "BTRFS entry invalid in this context.\n";
           defined $FAI::configs{BTRFS} or $FAI::configs{BTRFS}{volumes} = {};
@@ -836,6 +842,29 @@ $FAI::Parser = Parse::RecDescent->new(
           $FAI::partition_pointer_dev_name = "TMPFS$vol_id";
         }
         mountpoint tmpfs_size mount_options
+        | /^nfs\s+([^\s]+)\s+/
+        {
+          ($FAI::device eq "NFS") or die "nfs entry invalid in this context\n";
+          defined ($FAI::configs{NFS}) or &FAI::internal_error("NFS entry missing");
+
+          my $vol_id = 0;
+          foreach my $ex_vol_id (&FAI::numsort(keys %{ $FAI::configs{NFS}{volumes} })) {
+            defined ($FAI::configs{NFS}{volumes}{$ex_vol_id}{device}) or last;
+            $vol_id++;
+          }
+
+          my $nfssource = $1;
+          $nfssource =~ s/HOSTNAME/$ENV{'HOSTNAME'}/g;
+          $FAI::configs{NFS}{volumes}{$vol_id}{device} = $nfssource;
+          $FAI::configs{NFS}{volumes}{$vol_id}{filesystem} = "nfs";
+
+          # We don't do preserve for nfs
+          $FAI::configs{NFS}{volumes}{$vol_id}{preserve} = 0;
+
+          $FAI::partition_pointer = (\%FAI::configs)->{NFS}->{volumes}->{$vol_id};
+          $FAI::partition_pointer_dev_name = "NFS$vol_id";
+        }
+        mountpoint mount_options
         | type mountpoint size filesystem mount_options lv_or_fsopts
         | <error>
 
@@ -998,7 +1027,23 @@ $FAI::Parser = Parse::RecDescent->new(
                 $dev = "/dev/$dev";
               }
             }
-            my @candidates = glob($dev);
+            my @candidates;
+
+            # resolve /dev/disk/by-id symlinks
+            # those symlinks point to the actual device node
+            # partitions, which may not exist yet, are addressed
+            # by /dev/disk/by-id/${disk}-part${partno}
+            if ($dev =~ m{^(/dev/disk/by-id/.*?)(?:-part(\d+))?$}) {
+              my $part_no = $2;
+              my @real_candidates = map { Cwd::abs_path($_) } glob($1);
+              if (defined $part_no) {
+                @candidates = map { &FAI::make_device_name($_, $part_no) } @real_candidates;
+              } else {
+                @candidates = @real_candidates;
+              }
+            } else {
+              @candidates = glob($dev);
+            }
 
             # options are only valid for RAID
             defined ($opts) and ($FAI::device ne "RAID") and die "Option $opts invalid in a non-RAID context\n";
@@ -1241,6 +1286,15 @@ sub check_config {
         ($this_mp eq "none") or $all_mount_pts{$this_mp} = 1;
       }
       next;
+    } elsif ($config eq "NFS") {
+      foreach my $p (keys %{ $FAI::configs{$config}{volumes} }) {
+        my $this_mp = $FAI::configs{$config}{volumes}{$p}{mountpoint};
+        next if ($this_mp eq "-");
+        defined($all_mount_pts{$this_mp}) and die
+          "Mount point $this_mp used twice\n";
+        ($this_mp eq "none") or $all_mount_pts{$this_mp} = 1;
+      }
+      next;
     } elsif ($config eq "TMPFS") {
       foreach my $p (keys %{ $FAI::configs{$config}{volumes} }) {
         my $this_mp = $FAI::configs{$config}{volumes}{$p}{mountpoint};
